
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif.h"
#include "sif.h"

static void	Error(msg)
char		*msg;
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

static int	Ceil(d)
double		d;
{
	int	i=(int)d;

	return (d>0.0 && d>(double)i)?i+1:i;
}

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	Nx,Ny,BPS,Nh,Nv,y,x,v,h;
	Cell	**org,**new;
	double	m,xh,yv,ry,rx,o,ox,oy;
	char	*mae="memory allocation error.";

	if (argc!=4 && argc!=5)
	    Error("usage : bli orgTIFF multiple|(Mx|-Nx My|-Ny) newTIFF");

	ReadImageFile(argv[1],&Nx,&Ny,&BPS,&org,NULL);

#define PELS(arg,N)	(arg[0]!='-')?Ceil(atof(arg)*(double)N):-atoi(arg)

	if (argc==4) {
	    if ((m=atof(argv[2]))<1.0) Error("bad multiple of pixels.");

	    Nh=Ceil(m*(double)Nx);
	    Nv=Ceil(m*(double)Ny);
	}
	else
	    if ((Nh=PELS(argv[2],Nx))<Nx ||
		(Nv=PELS(argv[3],Ny))<Ny) Error("bad (multiple of) pixels.");

	xh=(double)Nx/(double)Nh;
	yv=(double)Ny/(double)Nv;

#define REALLOC(ptr,noe)	realloc(ptr,(size_t)(noe)*sizeof(*(ptr)))

	if ((org=(Cell **)REALLOC(org,Ny+2))==NULL) Error(mae);

	++org;

	if ((org[Ny]=(Cell *)calloc((size_t)(Nx+2),
				    sizeof(Cell)))==NULL) Error(mae);

	++org[Ny];

	for (y=Ny-1; y>=0; y--) {
	    if ((org[y]=(Cell *)REALLOC(org[y-1],Nx+2))==NULL) Error(mae);

	    ++org[y]; org[y][Nx]=0;
		      for (x=Nx-1; x>=0; x--) org[y][x]=org[y][x-1];
		      org[y][-1]=0;
	}
	org[-1]=org[Ny];

#define ALLOC(type,noe)	(type *)malloc((size_t)(noe)*sizeof(type))

	if ((new=ALLOC(Cell *,Nv))==NULL) Error(mae);

	for (y=(-1), v=0; v<Nv; v++) {
	    if ((new[v]=ALLOC(Cell,Nh))==NULL) Error(mae);

	    if ((ry=yv*((double)v+0.5)-(double)y+0.5)>=1.0) {
		++y; ry-=1.0;
	    }
	    for (x=(-1), h=0; h<Nh; h++) {
		if ((rx=xh*((double)h+0.5)-(double)x+0.5)>=1.0) {
		    ++x; rx-=1.0;
		}
		o=(double)org[y-1][x-1]; ox=(double)org[y-1][x]-o;
					 oy=(double)org[y][x-1]-o;
		new[v][h]=
		(Cell)(((double)org[y][x]-o-ox-oy)*rx*ry+ox*rx+oy*ry+o+0.5);
	    }
	}
	StoreImageFile(argv[argc-1],Nh,Nv,BPS,new,NULL);

	return 0;
}
