
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif.h"

static void	Error(msg)
char		*msg;
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

extern void	Case_1(),Case_3(),Case_9(),Case_7();

int	main(argc,argv)
int	argc;
char	**argv;
{
	int	Nx,Ny,x,y,sy,sx,S=1;
	Cell	**cell;
	double	base,step,**grid,**mesh,m,mx,my,mxy,ry,rx,
		Ux=1.0,Uy=1.0,L=0.0,A=0.0;
	char	*mae="memory allocation error.";

	if (argc!=3 && argc!=4 && argc!=5 && argc!=6)
	    Error("usage : l_a_bli TIFF {Ux Uy} {segment} threshold");

	ReadImageFile(argv[1],&Nx,&Ny,NULL,&cell,NULL);

	if (argc==5 || argc==6) {
	    Ux=atof(argv[2]);
	    Uy=atof(argv[3]);
	}
	if (argc==4 || argc==6) {
	    if ((S=atoi(argv[argc-2]))<1) Error("bad segment number.");

	    Ux/=(double)S;
	    Uy/=(double)S;
	}
	if (argv[argc-1][0]=='-') {
	    base=atof(argv[argc-1]+1); step=(-1.0);
	}
	else {
	    base=(-atof(argv[argc-1])); step=1.0;
	}

#define SIZE(type,noe)		sizeof(type)*(size_t)(noe)
#define REALLOC(type,ptr,noe)	(type *)realloc(ptr,SIZE(type,noe))
#define ALLOC(type,noe)		(type *)malloc(SIZE(type,noe))

	if ((cell=(Cell **)
	    (grid=REALLOC(double *,cell,Ny+2)))==NULL) Error(mae);

	++grid;

	if ((grid[Ny]=ALLOC(double,Nx+2))==NULL) Error(mae);

	++grid[Ny]; for (x=Nx; x>=(-1); x--) grid[Ny][x]=base;

	for (y=Ny-1; y>=0; y--) {
	    if ((cell[y]=(Cell *)
		(grid[y]=REALLOC(double,cell[y],Nx+2)))==NULL) Error(mae);

	    ++grid[y];
	    grid[y][Nx]=base;
	    for (x=Nx-1; x>=0; x--) grid[y][x]=base+step*(double)cell[y][x];
	    grid[y][-1]=base;
	}
	grid[-1]=grid[Ny];

	if ((mesh=ALLOC(double *,S+1))==NULL) Error(mae);

	for (sy=0; sy<=S; sy++)
	    if ((mesh[sy]=ALLOC(double,S+1))==NULL) Error(mae);

	for (y=0; y<=Ny; y++)
	for (x=0; x<=Nx; x++) {
	    m=grid[y-1][x-1];
	    mx=grid[y-1][x]-m;
	    my=grid[y][x-1]-m;
	    mxy=grid[y][x]-m-mx-my;
	    for (sy=0; sy<=S; sy++) { ry=(double)sy/(double)S;
	    for (sx=0; sx<=S; sx++) { rx=(double)sx/(double)S;
		mesh[sy][sx]=m+mx*rx+my*ry+mxy*rx*ry;
	    }}
	    for (sy=0; sy<S; sy++)
	    for (sx=0; sx<S; sx++)

#define C0	mesh[sy][sx]
#define C1	mesh[sy][sx+1]
#define C2	mesh[sy+1][sx]
#define C3	mesh[sy+1][sx+1]

		switch(((C0>=0.0)?1:0)|((C1>=0.0)?2:0)|
		       ((C2>=0.0)?4:0)|((C3>=0.0)?8:0)) {
		    case  1: Case_1 (Ux,Uy,C0,C1,C2,C3,&L,&A); break;
		    case  2: Case_1 (Ux,Uy,C1,C0,C3,C2,&L,&A); break;
		    case  4: Case_1 (Ux,Uy,C2,C3,C0,C1,&L,&A); break;
		    case  8: Case_1 (Ux,Uy,C3,C2,C1,C0,&L,&A); break;
		    case  3: Case_3 (Ux,Uy,C0,C1,C2,C3,&L,&A); break;
		    case  5: Case_3 (Uy,Ux,C0,C2,C1,C3,&L,&A); break;
		    case 10: Case_3 (Uy,Ux,C1,C3,C0,C2,&L,&A); break;
		    case 12: Case_3 (Ux,Uy,C3,C2,C1,C0,&L,&A); break;
		    case  6: Case_9 (Ux,Uy,C1,C0,C3,C2,&L,&A); break;
		    case  9: Case_9 (Ux,Uy,C0,C1,C2,C3,&L,&A); break;
		    case  7: Case_7 (Ux,Uy,C0,C1,C2,C3,&L,&A); break;
		    case 11: Case_7 (Ux,Uy,C1,C0,C3,C2,&L,&A); break;
		    case 13: Case_7 (Ux,Uy,C2,C3,C0,C1,&L,&A); break;
		    case 14: Case_7 (Ux,Uy,C3,C2,C1,C0,&L,&A); break;
		    case 15: A+=(Ux*Uy); break;
		}
	}
	(void)printf("%lf\t%lf\n",L,A);

	return 0;
}
